@@ -544,7 +544,7 @@ class VulnerabilityScoreSource(str, Enum):
         """
         if self == VulnerabilityScoreSource.OWASP:
             return 'OWASP Risk'
-        return self.value
+        return self.value  # type: ignore
 
 
 class VulnerabilitySeverity(str, Enum):
