@@ -19,20 +19,22 @@
 
 from itertools import chain
 from json import load as json_load
-from os.path import dirname, join as path_join
+from os.path import join as path_join
 from unittest import TestCase
 
 from ddt import data, ddt, idata, unpack
 
 from cyclonedx import spdx
 
-with open(path_join(dirname(__file__), '..', 'cyclonedx', 'schema', 'spdx.schema.json')) as spdx_schema:
+from . import CDX_SCHEMA_DIRECTORY
+
+with open(path_join(CDX_SCHEMA_DIRECTORY, 'spdx.schema.json')) as spdx_schema:
     KNOWN_SPDX_IDS = json_load(spdx_schema)['enum']
 
 VALID_COMPOUND_EXPRESSIONS = {
     # for valid test data see the spec: https://spdx.github.io/spdx-spec/v2.3/SPDX-license-expressions/
-    '(MIT WITH Apache-2.0)',
-    '(BSD-2-Clause OR Apache-2.0)',
+    '(MIT AND Apache-2.0)',
+    'BSD-2-Clause OR Apache-2.0',
 }
 
 
@@ -87,8 +89,9 @@ class TestSpdxIsCompoundExpression(TestCase):
         self.assertTrue(actual)
 
     @data(
+        'MIT AND Apache-2.0 OR something-unknown'
         'something invalid',
-        '(c) John Doe'
+        '(c) John Doe',
     )
     def test_negative(self, invalid_expression: str) -> None:
         actual = spdx.is_compound_expression(invalid_expression)
